"use strict";
Components.utils.import("resource://gre/modules/Services.jsm");

var redirectcleaner = {
	
	init: function() {
		
		var prefs = Services.prefs.getBranch("extensions.redirectcleaner.");
		
		redirectcleaner.set(prefs.getCharPref("whitelist"));
		
		document.getElementById("redirectcleaner-enable").checked = prefs.getBoolPref("enable");
		document.getElementById("redirectcleaner-target").checked = prefs.getBoolPref("target");
		document.getElementById("redirectcleaner-debug").checked = prefs.getBoolPref("debug");
		
	},
	
	accept: function() {
		
		var prefs = Services.prefs.getBranch("extensions.redirectcleaner.");
		
		prefs.setCharPref("whitelist", redirectcleaner.get());
		
		prefs.setBoolPref("enable", document.getElementById("redirectcleaner-enable").checked);
		prefs.setBoolPref("target", document.getElementById("redirectcleaner-target").checked);
		prefs.setBoolPref("debug", document.getElementById("redirectcleaner-debug").checked);
		
		window.opener.content.location.reload();
		
	},
	
	help: function() {
		
		window.openDialog("chrome://redirectcleaner/content/about.xul", "_blank", "chrome,dialog,modal,centerscreen", null);
		
	},
	
	export: function() {
		
		var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
		filePicker.init(window, "Redirect Cleaner", Components.interfaces.nsIFilePicker.modeSave);
		filePicker.appendFilter("TXT", "*.txt");
		filePicker.appendFilter("*", "*.*");
		filePicker.defaultExtension = "txt";
		filePicker.defaultString = "redirectcleaner.txt";
		
		if(filePicker.show() == Components.interfaces.nsIFilePicker.returnCancel) { return; }
		
		try {
			
			var fileOutputStream = Components.classes["@mozilla.org/network/file-output-stream;1"].createInstance(Components.interfaces.nsIFileOutputStream);
			fileOutputStream.init(filePicker.file, -1, 0, 0);
			
			var converterOutputStream = Components.classes["@mozilla.org/intl/converter-output-stream;1"].createInstance(Components.interfaces.nsIConverterOutputStream);
			converterOutputStream.init(fileOutputStream, "UTF-8", 0, 0);
			
			converterOutputStream.writeString(redirectcleaner.get());
			
			converterOutputStream.close();
			
		} catch(e) {
			
			alert("Redirect Cleaner: Error" + "\n" + e.message);
			return;
			
		}
		
	},
	
	import: function() {
		
		var filePicker = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
		filePicker.init(window, "Redirect Cleaner", Components.interfaces.nsIFilePicker.modeOpen);
		filePicker.appendFilter("TXT", "*.txt");
		filePicker.appendFilter("*", "*.*");
		
		if(filePicker.show() == Components.interfaces.nsIFilePicker.returnCancel) { return; }
		
		try {
			
			var fileInputStream = Components.classes["@mozilla.org/network/file-input-stream;1"].createInstance(Components.interfaces.nsIFileInputStream);
			fileInputStream.init(filePicker.file, -1, 0, 0);
			
			var converterInputStream = Components.classes["@mozilla.org/intl/converter-input-stream;1"].createInstance(Components.interfaces.nsIConverterInputStream);
			converterInputStream.init(fileInputStream, "UTF-8", 0, 0);
			
			var file = "", string = {};
			while(converterInputStream.readString(4096, string) != 0) {
				file += string.value;
			}
			
			redirectcleaner.set(redirectcleaner.get() + file);
			
			converterInputStream.close();
			
		} catch(e) {
			
			alert("Redirect Cleaner: Error" + "\n" + e.message);
			return;
			
		}
		
	},
	
	add: function() {
		
		var params = { accept: false, domain: null, link: null, regexp: null }
		window.openDialog("chrome://redirectcleaner/content/edit.xul", "_blank", "chrome,dialog,modal,centerscreen", params);
		if(params.accept) {
			
			if(params.domain || params.link || params.regexp) {
				
				redirectcleaner.set(redirectcleaner.get() + params.domain + "," + params.link + "," + params.regexp + ";");
				
			}
			
		}
		
	},
	
	edit: function() {
		
		var item = document.getElementById("redirectcleaner-whitelist").selectedItem.childNodes;
		
		var params = { accept: false, domain: item[0].getAttribute("label"), link: item[1].getAttribute("label"), regexp: item[2].getAttribute("label") }
		window.openDialog("chrome://redirectcleaner/content/edit.xul", "_blank", "chrome,dialog,modal,centerscreen", params);
		if(params.accept) {
			
			if(params.domain || params.link || params.regexp) {
				
				redirectcleaner.remove();
				redirectcleaner.set(redirectcleaner.get() + params.domain + "," + params.link + "," + params.regexp + ";");
				
			}
			
		}
		
	},
	
	remove: function() {
		
		var whitelist = document.getElementById("redirectcleaner-whitelist");
		whitelist.removeItemAt(whitelist.selectedIndex);
		
	},
	
	get: function() {
		
		var whitelist = document.getElementById("redirectcleaner-whitelist");
		
		var string = "";
		for(var i = 0; i < whitelist.itemCount; i++) {
			
			var item = whitelist.getItemAtIndex(i).childNodes;
			string += item[0].getAttribute("label") + "," + item[1].getAttribute("label") + "," + item[2].getAttribute("label") + ";";
			
		}
		
		return string;
		
	},
	
	set: function(string) {
		
		var whitelist = document.getElementById("redirectcleaner-whitelist");
		while(whitelist.itemCount) { whitelist.removeItemAt(0); }
		
		var items = string.split(";").sort();
		for(var i = 1; i < items.length; i++) {
			
			if(items[i] == items[i-1]) { continue; }
			
			var item = items[i].split(",");
			
			var listitem = document.createElement("listitem");
			listitem.setAttribute("ondblclick", "redirectcleaner.edit()");
			
			var listcell = document.createElement("listcell");
			listcell.setAttribute("label", item[0]);
			listitem.appendChild(listcell);
			
			var listcell = document.createElement("listcell");
			listcell.setAttribute("label", item[1]);
			listitem.appendChild(listcell);
			
			var listcell = document.createElement("listcell");
			listcell.setAttribute("label", item[2]);
			listcell.setAttribute("style", "text-align: center;");
			listitem.appendChild(listcell);
			
			var whitelist = document.getElementById("redirectcleaner-whitelist");
			whitelist.appendChild(listitem);
			
		}
		
	}
	
}
