"use strict";
Components.utils.import("resource://gre/modules/Services.jsm");

window.addEventListener("load", function() { redirectcleaner.load(); }, false);
window.addEventListener("unload", function() { redirectcleaner.unload(); }, false);

var redirectcleaner = {
	
	load: function() {
		
		redirectcleaner.prefs();
		Services.prefs.addObserver("extensions.redirectcleaner.", redirectcleaner, false);
		
		var contextmenu = document.getElementById("contentAreaContextMenu");
		contextmenu.addEventListener("popupshowing", function(event) { redirectcleaner.contextmenu(event); }, false);
		
		window.setTimeout(function() { redirectcleaner.delay(); }, 1000);
		
	},
	
	unload: function() {
		
		Services.prefs.removeObserver("extensions.redirectcleaner.", redirectcleaner, false);
		
	},
	
	prefs: function() {
		
		var prefs = Services.prefs.getBranch("extensions.redirectcleaner.");
		
		redirectcleaner.whitelist = prefs.getCharPref("whitelist");
		
		redirectcleaner.enable = prefs.getBoolPref("enable");
		redirectcleaner.target = prefs.getBoolPref("target");
		redirectcleaner.debug = prefs.getBoolPref("debug");
		
		if(prefs.getBoolPref("enable")) {
			
			var statusbar = document.getElementById("redirectcleaner-statusbar");
			if(statusbar) {
				
				statusbar.setAttribute("tooltiptext", "Redirect Cleaner: Enabled");
				statusbar.setAttribute("redirectcleaner", "enable");
				
			}
			
			var toolbar = document.getElementById("redirectcleaner-toolbar");
			if(toolbar) {
				
				toolbar.setAttribute("tooltiptext", "Redirect Cleaner: Enabled");
				toolbar.setAttribute("redirectcleaner", "enable");
				
			}
			
		}
		else {
			
			var statusbar = document.getElementById("redirectcleaner-statusbar");
			if(statusbar) {
				
				statusbar.setAttribute("tooltiptext", "Redirect Cleaner: Disabled");
				statusbar.setAttribute("redirectcleaner", "disable");
				
			}
			
			var toolbar = document.getElementById("redirectcleaner-toolbar");
			if(toolbar) {
				
				toolbar.setAttribute("tooltiptext", "Redirect Cleaner: Disabled");
				toolbar.setAttribute("redirectcleaner", "disable");
				
			}
			
		}
		
		if(prefs.getBoolPref("firstrun")) {
			
			var addonbar = document.getElementById("addon-bar");
			if(addonbar) {
				
				var currentset = addonbar.currentSet;
				if(currentset.indexOf("redirectcleaner-toolbar") == -1) {
					
					currentset += ",redirectcleaner-toolbar";
					
					addonbar.currentSet = currentset;
					addonbar.setAttribute("currentset", currentset);
					document.persist("addon-bar", "currentset");
					
				}
				
				prefs.setBoolPref("firstrun", false);
				
			}
			
		}
		
		if(prefs.getBoolPref("firstrun.v200")) {
			
			/* Redirect Cleaner 1.0.0 */
			prefs.setCharPref("whitelist", prefs.getCharPref("whitelist").replace(";", ",,;"));
			prefs.setBoolPref("firstrun.v200", false);
			
		}
		
	},
	
	delay: function() {
		
		if(window.caligon && window.caligon.status4evar && window.caligon.status4evar.statusService) {
			
			redirectcleaner.contentAreaClickOrig = contentAreaClick;
			contentAreaClick = redirectcleaner.contentAreaClick;
			
			redirectcleaner.setOverLinkOrig = window.caligon.status4evar.statusService.setOverLinkInternal;
			window.caligon.status4evar.statusService.setOverLinkInternal = redirectcleaner.setOverLink;
			
		}
		else {
			
			redirectcleaner.contentAreaClickOrig = contentAreaClick;
			contentAreaClick = redirectcleaner.contentAreaClick;
			
			redirectcleaner.setOverLinkOrig = XULBrowserWindow.setOverLink;
			XULBrowserWindow.setOverLink = redirectcleaner.setOverLink;
			
		}
		
	},
	
	contentAreaClick: function(event, click) {
		
		var element = event.target;
		do {
			
			if(element.tagName == "A") {
				
				redirectcleaner.check(element);
				break;
				
			}
			
		} while(element = element.parentElement);
		
		redirectcleaner.contentAreaClickOrig.call(this, event, click);
		
	},
	
	setOverLink: function(string, element) {
		
		if(!string || !element) { redirectcleaner.setOverLinkOrig.call(this, string, element); return; }
		
		redirectcleaner.check(element);
		redirectcleaner.setOverLinkOrig.call(this, element.href, element);
		
	},
	
	check: function(element) {
		
		/* whitelist */
		var items = redirectcleaner.whitelist.split(";").sort();
		for(var i = 1; i < items.length; i++) {
			
			try {
				
				var item = items[i].split(",");
				
				if(item[2] == "x" && content.location.href.match(new RegExp(item[0])) && element.href.match(new RegExp(item[1]))) {
					
					redirectcleaner.dump("Redirect Cleaner: Whitelist" + "\n\n" + "domain" + "\n" + content.location.href + "\n" + item[0] + "\n\n" + "link" + "\n" + element.href + "\n" + item[1] + "\n\n" + "regexp" + "\n" + item[2]);
					return;
					
				}
				
				if(item[2] != "x" && content.location.href.indexOf(item[0]) != -1 && element.href.indexOf(item[1]) != -1) {
					
					redirectcleaner.dump("Redirect Cleaner: Whitelist" + "\n\n" + "domain" + "\n" + content.location.href + "\n" + item[0] + "\n\n" + "link" + "\n" + element.href + "\n" + item[1] + "\n\n" + "regexp" + "\n" + item[2]);
					return;
					
				}
				
			} catch(e) {
			}
			
		}
		
		if(redirectcleaner.enable) {
			
			/* href */
			if(element.hasAttribute("href")) {
				
				var href = element.getAttribute("href");
				var clean = redirectcleaner.clean(redirectcleaner.javascript(href));
				
				if(href != clean) {
					
					redirectcleaner.dump("Redirect Cleaner" + "\n\n" + "href" + "\n" + href + "\n\n" + "redirectcleaner" + "\n" + clean);
					
					element.setAttribute("redirectcleaner", element.href);
					element.setAttribute("redirectcleaner-href", href);
					element.setAttribute("href", clean);
					
				}
				
			}
			
			/* onclick */
			if(element.hasAttribute("onclick")) {
				
				var href = element.getAttribute("href");
				var onclick = element.getAttribute("onclick");
				var clean = redirectcleaner.javascript(onclick);
				
				if(onclick != clean) {
					
					redirectcleaner.dump("Redirect Cleaner" + "\n\n" + "href" + "\n" + href + "\n\n" + "onclick" + "\n" + onclick + "\n\n" + "redirectcleaner" + "\n" + clean);
					
					element.setAttribute("redirectcleaner-onclick", onclick);
					element.removeAttribute("onclick");
					
					if(!clean.match("outbrain.com")) {
						
						element.setAttribute("redirectcleaner", element.href);
						element.setAttribute("redirectcleaner-href", href);
						element.setAttribute("href", clean);
						
					}
					
				}
				
			}
			
			/* onkeydown */
			if(element.hasAttribute("onkeydown")) {
				
				var href = element.getAttribute("href");
				var onkeydown = element.getAttribute("onkeydown");
				var clean = redirectcleaner.javascript(onkeydown);
				
				if(onkeydown != clean) {
					
					redirectcleaner.dump("Redirect Cleaner" + "\n\n" + "href" + "\n" + href + "\n\n" + "onkeydown" + "\n" + onkeydown + "\n\n" + "redirectcleaner" + "\n" + clean);
					
					element.setAttribute("redirectcleaner-onkeydown", onkeydown);
					element.removeAttribute("onkeydown");
					
					if(!clean.match("outbrain.com")) {
						
						element.setAttribute("redirectcleaner", element.href);
						element.setAttribute("redirectcleaner-href", href);
						element.setAttribute("href", clean);
						
					}
					
				}
				
			}
			
			/* onmousedown */
			if(element.hasAttribute("onmousedown")) {
				
				var href = element.getAttribute("href");
				var onmousedown = element.getAttribute("onmousedown");
				var clean = redirectcleaner.javascript(onmousedown);
				
				if(onmousedown != clean) {
					
					redirectcleaner.dump("Redirect Cleaner" + "\n\n" + "href" + "\n" + href + "\n\n" + "onmousedown" + "\n" + onmousedown + "\n\n" + "redirectcleaner" + "\n" + clean);
					
					element.setAttribute("redirectcleaner-onmousedown", onmousedown);
					element.removeAttribute("onmousedown");
					
					if(!clean.match("outbrain.com")) {
						
						element.setAttribute("redirectcleaner", element.href);
						element.setAttribute("redirectcleaner-href", href);
						element.setAttribute("href", clean);
						
					}
					
				}
				
			}
			
			/* target */
			if(element.hasAttribute("target") && redirectcleaner.target) {
				
				redirectcleaner.dump("Redirect Cleaner" + "\n\n" + "target" + "\n" + element.getAttribute("target"));
				
				element.setAttribute("redirectcleaner-target", element.getAttribute("target"));
				element.removeAttribute("target");
				
			}
			
		}
		else {
			
			/* redirectcleaner */
			if(element.hasAttribute("redirectcleaner")) {
				
				element.removeAttribute("redirectcleaner");
				
			}
			
			/* href */
			if(element.hasAttribute("redirectcleaner-href")) {
				
				element.setAttribute("href", element.getAttribute("redirectcleaner-href"));
				element.removeAttribute("redirectcleaner-href");
				
			}
			
			/* onclick */
			if(element.hasAttribute("redirectcleaner-onclick")) {
				
				element.setAttribute("onclick", element.getAttribute("redirectcleaner-onclick"));
				element.removeAttribute("redirectcleaner-onclick");
				
			}
			
			/* onkeydown */
			if(element.hasAttribute("redirectcleaner-onkeydown")) {
				
				element.setAttribute("onkeydown", element.getAttribute("redirectcleaner-onkeydown"));
				element.removeAttribute("redirectcleaner-onkeydown");
				
			}
			
			/* onmousedown */
			if(element.hasAttribute("redirectcleaner-onmousedown")) {
				
				element.setAttribute("onmousedown", element.getAttribute("redirectcleaner-onmousedown"));
				element.removeAttribute("redirectcleaner-onmousedown");
				
			}
			
			/* target */
			if(element.hasAttribute("redirectcleaner-target")) {
				
				element.setAttribute("target", element.getAttribute("redirectcleaner-target"));
				element.removeAttribute("redirectcleaner-target");
				
			}
			
		}
		
	},
	
	clean: function(url) {
		
		/* whitelist */
		var objPattern = /(?:magnet|subscribe|login|logout|register|signin|signout|signon|signoff|signup)/i;
		if(objPattern.test(url)) {
			
			return url;
			
		}
		
		/* www */
		var objPattern = /[?&=](?:((?:www)[.][^?&;]*[?][^?]*)$|((?:www)[.][^?&;]*))/i;
		var arrMatches = objPattern.exec(url);
		if(arrMatches) {
			
			var value = (arrMatches[1] || arrMatches[2]);
			
			if(value.match(/(?:%25|%2F|%3F|%26|%3D)/i)) {
				value = decodeURIComponent(value);
			}
			
			return redirectcleaner.clean("http://" + value);
			
		}
		
		/* http */
		var objPattern = /[/?&=*+-](?:((?:ftp|http|https)[:][^?&;]*[?][^?]*)$|((?:ftp|http|https)[:][^?&;]*))/i;
		var arrMatches = objPattern.exec(url);
		if(arrMatches) {
			
			var value = (arrMatches[1] || arrMatches[2]);
			
			if(value.match(/(?:%25|%2F|%3F|%26|%3D)/i)) {
				value = decodeURIComponent(value);
			}
			
			return redirectcleaner.clean(value);
			
		}
		
		/* http */
		var objPattern = /[?&=*+-](?:((?:ftp|http|https)(?:%3A)[^?&;]*[?][^?]*)$|((?:ftp|http|https)(?:%3A)[^?&;]*))/i;
		var arrMatches = objPattern.exec(url);
		if(arrMatches) {
			
			var value = (arrMatches[1] || arrMatches[2]);
			return redirectcleaner.clean(decodeURIComponent(value));
			
		}
		
		/* http */
		var objPattern = /[?&=*+-](?:((?:ftp|http|https)(?:%253A)[^?&;]*[?][^?]*)$|((?:ftp|http|https)(?:%253A)[^?&;]*))/i;
		var arrMatches = objPattern.exec(url);
		if(arrMatches) {
			
			var value = (arrMatches[1] || arrMatches[2]);
			return redirectcleaner.clean(decodeURIComponent(decodeURIComponent(value)));
			
		}
		
		/* base64 */
		var objPattern = /[?&=~]((?:ZnRw|aHR0c)[^?&;~]*)/i;
		var arrMatches = objPattern.exec(url);
		if(arrMatches) {
			
			try {
				
				var value = atob(decodeURIComponent(arrMatches[1]));
				
				if(value.match(/(?:%25|%2F|%3F|%26|%3D)/i)) {
					value = decodeURIComponent(value);
				}
				
				return redirectcleaner.clean(value);
				
			} catch(e) {
			}
			
		}
		
		return url;
		
	},
	
	javascript: function(url) {
		
		/* whitelist */
		var objPattern = /(?:s_objectID)/i;
		if(objPattern.test(url)) {
			
			return url;
			
		}
		
		/* javascript */
		var objPattern = /(?:[']((?:ftp|http|https)[:][^']*)[']|["]((?:ftp|http|https)[:][^"]*)["])/i;
		var arrMatches = objPattern.exec(url);
		if(arrMatches) {
			
			var value = (arrMatches[1] || arrMatches[2]);
			return redirectcleaner.clean(value);
			
		}
		
		return url;
		
	},
	
	contextmenu: function(event) {
		
		switch(event.type) {
			
			case "popupshowing":
				
				if(gContextMenu && gContextMenu.link && gContextMenu.link.hasAttribute("redirectcleaner")) {
					
					var contextmenu = document.getElementById("redirectcleaner-contextmenu");
					contextmenu.setAttribute("tooltiptext", gContextMenu.link.getAttribute("redirectcleaner"));
					contextmenu.hidden = false;
					
					var separator = document.getElementById("redirectcleaner-separator");
					separator.hidden = false;
					
				}
				else {
					
					var contextmenu = document.getElementById("redirectcleaner-contextmenu");
					contextmenu.setAttribute("tooltiptext", "");
					contextmenu.hidden = true;
					
					var separator = document.getElementById("redirectcleaner-separator");
					separator.hidden = true;
					
				}
				
				break;
				
			case "command":
				
				window.openUILink(gContextMenu.link.getAttribute("redirectcleaner"), event);
				break;
				
			default: ;
			
		}
		
	},
	
	statusbar: function(event) {
		
		switch(event.button) {
			
			case 0:
				
				var prefs = Services.prefs.getBranch("extensions.redirectcleaner.");
				prefs.setBoolPref("enable", !prefs.getBoolPref("enable"));
				break;
				
			case 1:
				
				var params = { accept: false, domain: content.location.hostname, link: null, regexp: null }
				window.openDialog("chrome://redirectcleaner/content/edit.xul", "_blank", "chrome,dialog,modal,centerscreen", params);
				if(params.accept) {
					
					if(params.domain || params.link || params.regexp) {
						
						if(redirectcleaner.whitelist.indexOf(params.domain + "," + params.link + "," + params.regexp + ";") != -1) { return; }
						
						var prefs = Services.prefs.getBranch("extensions.redirectcleaner.");
						prefs.setCharPref("whitelist", prefs.getCharPref("whitelist") + params.domain + "," + params.link + "," + params.regexp + ";");
						
						content.location.reload();
						
					}
					
				}
				
				break;
				
			case 2:
				
				window.openDialog("chrome://redirectcleaner/content/options.xul", "_blank", "chrome,dialog,modal,centerscreen", null);
				break;
				
			default: ;
			
		}
		
	},
	
	toolbar: function(event) {
		
		switch(event.button) {
			
			case 0:
				
				var prefs = Services.prefs.getBranch("extensions.redirectcleaner.");
				prefs.setBoolPref("enable", !prefs.getBoolPref("enable"));
				break;
				
			case 1:
				
				var params = { accept: false, domain: content.location.hostname, link: null, regexp: null }
				window.openDialog("chrome://redirectcleaner/content/edit.xul", "_blank", "chrome,dialog,modal,centerscreen", params);
				if(params.accept) {
					
					if(params.domain || params.link || params.regexp) {
						
						if(redirectcleaner.whitelist.indexOf(params.domain + "," + params.link + "," + params.regexp + ";") != -1) { return; }
						
						var prefs = Services.prefs.getBranch("extensions.redirectcleaner.");
						prefs.setCharPref("whitelist", prefs.getCharPref("whitelist") + params.domain + "," + params.link + "," + params.regexp + ";");
						
						content.location.reload();
						
					}
					
				}
				
				break;
				
			case 2:
				
				window.openDialog("chrome://redirectcleaner/content/options.xul", "_blank", "chrome,dialog,modal,centerscreen", null);
				break;
				
			default: ;
			
		}
		
	},
	
	observe: function(subject, topic, data) {
		
		if(topic != "nsPref:changed") { return; }
		redirectcleaner.prefs();
		
	},
	
	dump: function(string) {
		
		if(!redirectcleaner.debug) { return; }
		Services.console.logStringMessage(string);
		
	}
	
}
